# -*- coding: utf-8 -*-

from django import forms
from django.core.validators import RegexValidator, URLValidator, validate_email
from PIL import Image

from portailuser.fields import PortailUserPhoneField

from gestion.eleves.forms import TAILLE_PHOTO, RATIO_PHOTO

INFOS_CONTACT = {
    "skype": (
        "Skype",
        "Pseudo",
        RegexValidator(r"^[a-zA-Z][a-zA-Z0-9.,-_]{5,31}$", "Pseudo invalide."),
    ),
    "personalWebsite": ("Sites Web perso (hors INP-net)", "Adresse", URLValidator()),
    "mailAnnexe": ("E-mail", "Adresse", validate_email),
}


class MotDePasseForm(forms.Form):
    original = forms.CharField(label="Mot de passe original", widget=forms.PasswordInput(render_value=False))
    nouveau = forms.CharField(label="Nouveau mot de passe", widget=forms.PasswordInput(render_value=False))
    confirmation = forms.CharField(label="Confirmation", widget=forms.PasswordInput(render_value=False))

    def __init__(self, ldap_user, *args, **kwargs):
        super(MotDePasseForm, self).__init__(*args, **kwargs)
        self.ldap_user = ldap_user

    def clean_original(self):
        original = self.cleaned_data["original"]

        if not self.ldap_user.try_login(original):
            raise forms.ValidationError("Le mot de passe original est " "incorrect.")

        return original

    def clean_confirmation(self):
        nouveau = self.cleaned_data.get("nouveau")
        confirmation = self.cleaned_data.get("confirmation")

        if nouveau is None or nouveau != confirmation:
            raise forms.ValidationError("Le nouveau mot de passe et la " "confirmation ne correspondent pas.")

        return confirmation


class InfosPersoForm(forms.Form):
    nickname = forms.CharField(
        label="Surnom",
        required=False,
        help_text="Utilisé notamment lors de votre connexion aux salons de " "discussion.",
    )
    homePhone = PortailUserPhoneField(label="Téléphone fixe", required=False)
    mobile = PortailUserPhoneField(label="Téléphone portable", required=False)
    postalAddress = forms.CharField(
        label="Adresse personnelle",
        required=False,
        widget=forms.Textarea(attrs={"rows": "4", "cols": "40"}),
    )
    parentsAddress = forms.CharField(
        label="Adresse des parents",
        required=False,
        widget=forms.Textarea(attrs={"rows": "4", "cols": "40"}),
        help_text="Ne sera pas affichée sur le portail.",
    )

    def __init__(self, *args, **kwargs):
        super(InfosPersoForm, self).__init__(*args, **kwargs)


class PhotosForm(forms.Form):
    chg_photo = forms.FileField(label="Nouvelle photo")

    def orient_photo(self, image):
        try:
            # Extraction du dictionnaire des métadonnées EXIF
            exif = image._getexif()
        except AttributeError:
            return None

        # Vérification de l'existence de l'EXIF, même si l'image est un JPEG
        if exif is None:
            return None

        # Vérification si l'image est un JPEG et donc si elle a des EXIF
        try:
            # Extraction de la métadonnée EXIF orientation de la photo, qui a pour key 274
            orientation = exif[274]
        except KeyError:
            return None

        return orientation

    def clean_chg_photo(self):
        chg_photo = self.cleaned_data["chg_photo"]

        if chg_photo is None:
            return chg_photo

        try:
            image = Image.open(chg_photo)
            image.load()
        except IOError:
            raise forms.ValidationError("Le fichier envoyé n’est pas une " "image valide.")

        orientation = self.orient_photo(image)

        # Si l'image est tournée d'un quart de tour dans le sens horaire
        if orientation == 6:
            image = image.rotate(-90)
            # Si l'image est tournée d'un quart de tour dans le sens anti-horaire
        elif orientation == 8:
            image = image.rotate(90)
            # Si l'image est à l'envers, vous êtes une bite

        larg, haut = image.size
        larg_alt = int(round(haut * RATIO_PHOTO))
        haut_alt = int(round(larg / RATIO_PHOTO))

        if larg_alt <= larg:
            decoupe = (larg - larg_alt) / 2
            image = image.crop((decoupe, 0, larg - decoupe, haut))
        else:
            decoupe = (haut - haut_alt) / 2
            image = image.crop((0, decoupe, larg, haut - decoupe))

        image = image.resize(TAILLE_PHOTO, Image.BILINEAR)

        self.chg_photo_image = image

        return chg_photo


class InfosContactForm(forms.Form):
    def __init__(self, ldap_user, donnees_post=None, infos=INFOS_CONTACT, **kwargs):
        self.bouton_presse = False

        # On récupère les données (soit par POST, soit par le LDAP)
        if donnees_post:
            donnees = {}
            for (ident, (titre, nom, validateur)) in infos.items():
                cpt = 1
                donnees[ident] = []
                while (ident + str(cpt)) in donnees_post:
                    if ("moins_" + ident + str(cpt)) in donnees_post:
                        self.bouton_presse = True
                        cpt += 1
                        continue

                    donnees[ident].append(donnees_post[ident + str(cpt)])
                    cpt += 1

                if ("plus_" + ident) in donnees_post:
                    self.bouton_presse = True
                    donnees[ident].append("")

            # Il ne faut pas passer un dictionnaire vide (sinon il sera ignoré)
            super(InfosContactForm, self).__init__({"x": ""}, **kwargs)
            del self.data["x"]
        else:
            donnees = {}

            for champ in infos.keys():
                donnees[champ] = getattr(ldap_user, champ)

            super(InfosContactForm, self).__init__(**kwargs)

        # On peut maintenant créer les champs du formulaire
        for (ident, (titre, nom, validateur)) in infos.items():
            # Au moins un champ par catégorie
            if (ident not in donnees.keys()) or (len(donnees[ident]) == 0):
                donnees[ident] = [""]

            validateurs = [validateur] if validateur else []
            cpt = 1
            for valeur in donnees[ident]:
                champ = forms.CharField(
                    label=nom + " " + str(cpt),
                    validators=validateurs,
                    required=False,
                    widget=forms.TextInput(attrs={"size": "32"}),
                )
                champ.a_bouton_plus = False
                champ.a_bouton_moins = True
                if cpt == 1:
                    champ.titre = titre  # Titre de la section
                champ.ident_interne = ident

                # Rentrée des valeurs :
                if donnees_post:
                    # Dans les données “POST” (active la validation des champs)
                    self.data[ident + str(cpt)] = valeur
                else:
                    # Par valeur initiale
                    champ.initial = valeur

                self.fields[ident + str(cpt)] = champ
                cpt += 1

            # Bouton “plus” sur le dernier champ
            champ.a_bouton_plus = True
            # Bouton “moins” s’il n’est pas tout seul, ou qu’il n’est pas vide
            champ.a_bouton_moins = len(donnees[ident]) > 1 or ident in donnees.keys()

    def donnees_entrees(self):
        donnees = {}
        for (ident_champ, valeur) in self.cleaned_data.items():
            if not valeur:
                continue

            ident = self.fields[ident_champ].ident_interne

            try:
                donnees[ident].append(valeur)
            except KeyError:
                donnees[ident] = [valeur]

        return donnees


class AdrRedirForm(forms.Form):
    mailForwardingAddress = forms.EmailField(
        label="E-mail de redirection",
        widget=forms.TextInput(attrs={"size": "32"}),
        required=False,
    )
    mdp = forms.CharField(
        label="Mot de passe",
        widget=forms.PasswordInput(render_value=False),
        help_text="Par mesure de sécurité, vous devez entrer à nouveau votre "
        "mot de passe INP-net pour modifier votre adresse e-mail de "
        "redirection.",
    )

    def __init__(self, ldap_user, *args, **kwargs):
        super(AdrRedirForm, self).__init__(*args, **kwargs)
        self.ldap_user = ldap_user

    def clean_mdp(self):
        mdp = self.cleaned_data["mdp"]

        if not self.ldap_user.try_login(mdp):
            raise forms.ValidationError("Ce mot de passe est incorrect.")

        return mdp
